<?php
/**
 * @license
 * Copyright 2018 TruongLuu. All Rights Reserved.
 */

namespace Truonglv\StickyTimer\XF\Pub\Controller;

use XF\Mvc\ParameterBag;
use Truonglv\StickyTimer\Listener;
use Truonglv\StickyTimer\Entity\Sticky;

/**
 * Class Thread
 * @package Truonglv\StickyTimer\XF\Pub\Controller
 * @inheritdoc
 */
class Thread extends XFCP_Thread
{
    public function actionStickyTimer(ParameterBag $params)
    {
        $thread = $this->assertViewableThread($params->thread_id);

        if (!Listener::isEnableNode($thread->node_id)) {
            return $this->noPermission();
        }

        if (!$thread->canStickUnstick($error)) {
            return $this->noPermission($error);
        }

        if ($this->isPost()) {
            $input = $this->filter([
                'expire_type' => 'bool',
                'expire' => [
                    'amount' => 'uint',
                    'unit' => 'str'
                ]
            ]);

            $expireDate = 0;
            if ($input['expire_type']) {
                if (!isset(Listener::$timeUnits[$input['expire']['unit']])) {
                    return $this->error(\XF::phrase('please_enter_valid_value'));
                }

                $expireDate = \XF::$time + $input['expire']['amount'] * Listener::$timeUnits[$input['expire']['unit']];
            }

            /** @var Sticky $sticky */
            $sticky = $thread->getRelationOrDefault('StickyTimer', false);
            $sticky->thread_id = $thread->thread_id;
            $sticky->expire_date = (int) $expireDate;

            $thread->sticky = true;
            $sticky->addCascadedSave($thread);

            $sticky->save();

            return $this->redirect($this->buildLink('threads', $thread));
        }

        $sticky = $thread->getRelation('StickyTimer');

        $expireType = 1;
        $amount = 0;
        $unit = 'hours';

        if ($sticky) {
            if ($sticky->expire_date === 0) {
                $expireType = 0;
            } else {
                $time = $sticky->expire_date - \XF::$time;

                $units = Listener::$timeUnits;
                arsort($units, SORT_NUMERIC);

                foreach ($units as $unitStr_ => $amountInt_) {
                    if (floor($time / $amountInt_) < 1) {
                        continue;
                    }

                    $amount = ceil($time / $amountInt_);
                    $unit = $unitStr_;

                    break;
                }
            }
        }

        $viewParams = [
            'thread' => $thread,
            'sticky' => $sticky,
            'expireType' => $expireType,
            'amount' => $amount,
            'unit' => $unit
        ];

        return $this->view(
            'Truonglv\StickyTimer:Thread\Sticky',
            'tl_thread_sticky_timer',
            $viewParams
        );
    }
}
